/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.pulsar;

import java.util.Comparator;
import java.util.List;
import org.testng.IMethodInstance;
import org.testng.IMethodInterceptor;
import org.testng.ITestContext;

// Sorts the test methods by test object instance hashcode, then priority, then method name. Useful when Factory
// generated tests interfere with each other.
public class TestNGInstanceOrder implements IMethodInterceptor {
    @Override
    public List<IMethodInstance> intercept(List<IMethodInstance> methods, ITestContext context) {
        return methods.stream().sorted(Comparator.<IMethodInstance>comparingInt(o -> o.getInstance().hashCode())
                        .thenComparingInt(o -> o.getMethod().getInterceptedPriority())
                        .thenComparingInt(o -> o.getMethod().getPriority())
                        .thenComparing(o -> o.getMethod().getMethodName()))
                .toList();
    }
}
